@@ -28,7 +28,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <string.h>
 
@@ -64,8 +64,8 @@ struct _GtkAboutDialogPrivate
   gchar *version;
   gchar *copyright;
   gchar *comments;
-  gchar *website;
-  gchar *website_label;
+  gchar *website_url;
+  gchar *website_text;
   gchar *translator_credits;
   gchar *license;
   
@@ -78,6 +78,7 @@ struct _GtkAboutDialogPrivate
   GtkWidget *comments_label;
   GtkWidget *copyright_label;
   GtkWidget *website_button;
+  GtkWidget *website_label;
 
   GtkWidget *credits_button;
   GtkWidget *credits_dialog;
@@ -124,6 +125,7 @@ static void                 gtk_about_dialog_set_property   (GObject
 							     guint               prop_id,
 							     const GValue       *value,
 							     GParamSpec         *pspec);
+static void                 gtk_about_dialog_show           (GtkWidget          *widge);
 static void                 update_name_version             (GtkAboutDialog     *about);
 static GtkIconSet *         icon_set_new_from_pixbufs       (GList              *pixbufs);
 static void                 activate_url                    (GtkWidget          *widget,
@@ -182,6 +184,7 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 
   object_class->finalize = gtk_about_dialog_finalize;
 
+  widget_class->show = gtk_about_dialog_show;
 
   /**
    * GtkAboutDialog:program-name:
@@ -433,8 +436,8 @@ gtk_about_dialog_init (GtkAboutDialog *about)
   priv->version = NULL;
   priv->copyright = NULL;
   priv->comments = NULL;
-  priv->website = NULL;
-  priv->website_label = NULL;
+  priv->website_url = NULL;
+  priv->website_text = NULL;
   priv->translator_credits = NULL;
   priv->license = NULL;
   priv->authors = NULL;
@@ -477,14 +480,19 @@ gtk_about_dialog_init (GtkAboutDialog *about)
   gtk_label_set_justify (GTK_LABEL (priv->copyright_label), GTK_JUSTIFY_CENTER);
   gtk_box_pack_start (GTK_BOX (vbox), priv->copyright_label, FALSE, FALSE, 0);
 
-  button = gtk_link_button_new (""); 
+  hbox = gtk_hbox_new (TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, FALSE, 0); 
+
+  priv->website_button = button = gtk_link_button_new (""); 
+  gtk_widget_set_no_show_all (button, TRUE);
   g_signal_connect (G_OBJECT (button), "clicked",
 		    G_CALLBACK (activate_url), about);
-
-  hbox = gtk_hbox_new (TRUE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, FALSE, 0); 
-  priv->website_button = button;
+  
+  priv->website_label = button = gtk_label_new ("");
+  gtk_widget_set_no_show_all (button, TRUE);
+  gtk_label_set_selectable (GTK_LABEL (button), TRUE);
+  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   
   gtk_widget_show (vbox);
   gtk_widget_show (priv->logo_image);
@@ -505,7 +513,8 @@ gtk_about_dialog_init (GtkAboutDialog *about)
   gtk_box_pack_end (GTK_BOX (GTK_DIALOG (about)->action_area), 
 		    button, FALSE, TRUE, 0); 
   gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (GTK_DIALOG (about)->action_area), button, TRUE);
-  g_signal_connect (button, "clicked", G_CALLBACK (display_credits_dialog), about);
+  g_signal_connect (button, "clicked",
+                    G_CALLBACK (display_credits_dialog), about);
   priv->credits_button = button;
   priv->credits_dialog = NULL;
 
@@ -516,7 +525,8 @@ gtk_about_dialog_init (GtkAboutDialog *about)
   gtk_box_pack_end (GTK_BOX (GTK_DIALOG (about)->action_area), 
 		    button, FALSE, TRUE, 0); 
   gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (GTK_DIALOG (about)->action_area), button, TRUE);
-  g_signal_connect (button, "clicked", G_CALLBACK (display_license_dialog), about);
+  g_signal_connect (button, "clicked",
+                    G_CALLBACK (display_license_dialog), about);
   priv->license_button = button;
   priv->license_dialog = NULL;
 
@@ -543,8 +553,8 @@ gtk_about_dialog_finalize (GObject *object)
   g_free (priv->copyright);
   g_free (priv->comments);
   g_free (priv->license);
-  g_free (priv->website);
-  g_free (priv->website_label);
+  g_free (priv->website_url);
+  g_free (priv->website_text);
   g_free (priv->translator_credits);
 
   g_strfreev (priv->authors);
@@ -643,10 +653,10 @@ gtk_about_dialog_get_property (GObject    *object,
       g_value_set_string (value, priv->comments);
       break;
     case PROP_WEBSITE:
-      g_value_set_string (value, priv->website);
+      g_value_set_string (value, priv->website_url);
       break;
     case PROP_WEBSITE_LABEL:
-      g_value_set_string (value, priv->website_label);
+      g_value_set_string (value, priv->website_text);
       break;
     case PROP_LICENSE:
       g_value_set_string (value, priv->license);
@@ -689,6 +699,44 @@ gtk_about_dialog_get_property (GObject    *object,
     }
 }
 
+static void
+update_website (GtkAboutDialog *about)
+{
+  GtkAboutDialogPrivate *priv = (GtkAboutDialogPrivate *)about->private_data;
+
+  if (priv->website_url && activate_url_hook)
+    {
+      gtk_widget_show (priv->website_button);
+      gtk_widget_hide (priv->website_label);
+
+      gtk_link_button_set_uri (GTK_LINK_BUTTON (priv->website_button), priv->website_url);
+      if (priv->website_text)
+        gtk_button_set_label (GTK_BUTTON (priv->website_button), priv->website_text);
+      else 
+        gtk_button_set_label (GTK_BUTTON (priv->website_button), priv->website_url);
+    }
+  else
+    {
+      gtk_widget_show (priv->website_label);
+      gtk_widget_hide (priv->website_button);
+
+      if (priv->website_url)
+        gtk_label_set_text (GTK_LABEL (priv->website_label), priv->website_url);
+      else if (priv->website_text)
+        gtk_label_set_text (GTK_LABEL (priv->website_label), priv->website_text);
+      else
+        gtk_widget_hide (priv->website_label);
+    }
+}
+
+static void
+gtk_about_dialog_show (GtkWidget *widget)
+{
+  update_website (GTK_ABOUT_DIALOG (widget));
+  
+  GTK_WIDGET_CLASS (gtk_about_dialog_parent_class)->show (widget);
+}
+
 /**
  * gtk_about_dialog_get_name:
  * @about: a #GtkAboutDialog
@@ -700,7 +748,7 @@ gtk_about_dialog_get_property (GObject    *object,
  *
  * Since: 2.6
  *
- * @Deprecated: 2.12: Use gtk_about_dialog_get_program_name() instead.
+ * Deprecated: 2.12: Use gtk_about_dialog_get_program_name() instead.
  **/
 G_CONST_RETURN gchar *
 gtk_about_dialog_get_name (GtkAboutDialog *about)
@@ -765,7 +813,7 @@ update_name_version (GtkAboutDialog *about)
  * 
  * Since: 2.6
  *
- * @Deprecated: 2.12: Use gtk_about_dialog_set_program_name() instead.
+ * Deprecated: 2.12: Use gtk_about_dialog_set_program_name() instead.
  **/
 void
 gtk_about_dialog_set_name (GtkAboutDialog *about, 
@@ -1115,7 +1163,7 @@ gtk_about_dialog_get_website (GtkAboutDialog *about)
 
   priv = (GtkAboutDialogPrivate *)about->private_data;
 
-  return priv->website;
+  return priv->website_url;
 }
 
 /**
@@ -1125,6 +1173,9 @@ gtk_about_dialog_get_website (GtkAboutDialog *about)
  * 
  * Sets the URL to use for the website link.
  *
+ * Note that that the hook functions need to be set up
+ * before calling this function.
+ *
  * Since: 2.6
  **/
 void
@@ -1138,32 +1189,11 @@ gtk_about_dialog_set_website (GtkAboutDialog *about,
 
   priv = (GtkAboutDialogPrivate *)about->private_data;
   
-  tmp = priv->website;
-  if (website != NULL)
-    {
-      priv->website = g_strdup (website);
-      if (activate_url_hook != NULL)
-	{
-	  gtk_link_button_set_uri (GTK_LINK_BUTTON (priv->website_button), website);  
-	  if (priv->website_label == NULL) 
-	    gtk_about_dialog_set_website_label (about, website);
-	}
-      else 
-	{
-	  GtkWidget *hbox = priv->website_button->parent;
-	  gtk_widget_destroy (priv->website_button);
-	  priv->website_button = gtk_label_new (website);
-	  gtk_label_set_selectable (GTK_LABEL (priv->website_button), TRUE);
-	  gtk_container_add (GTK_CONTAINER (hbox), priv->website_button);
-	  gtk_widget_show (priv->website_button);
-	}
-    }
-  else 
-    {
-      priv->website = NULL;
-      gtk_widget_hide (priv->website_button);
-    }
-  g_free (tmp);
+  tmp = priv->website_url;
+  priv->website_url = g_strdup (website);
+  g_free (tmp); 
+
+  update_website (about);
 
   g_object_notify (G_OBJECT (about), "website");
 }
@@ -1188,7 +1218,7 @@ gtk_about_dialog_get_website_label (GtkAboutDialog *about)
 
   priv = (GtkAboutDialogPrivate *)about->private_data;
 
-  return priv->website_label;
+  return priv->website_text;
 }
 
 /**
@@ -1212,24 +1242,12 @@ gtk_about_dialog_set_website_label (GtkAboutDialog *about,
 
   priv = (GtkAboutDialogPrivate *)about->private_data;
 
-  tmp = priv->website_label;
-  if (activate_url_hook != NULL)
-    {
-      if (website_label != NULL) 
-	{
-	  priv->website_label = g_strdup (website_label);
-	  gtk_button_set_label (GTK_BUTTON (priv->website_button),
-				priv->website_label);
-	  gtk_widget_show (priv->website_button);
-	}
-      else 
-	{
-	  priv->website_label = NULL;
-	  gtk_widget_hide (priv->website_button);
-	}
-    }
+  tmp = priv->website_text;
+  priv->website_text = g_strdup (website_label);
   g_free (tmp);
 
+  update_website (about);
+
   g_object_notify (G_OBJECT (about), "website-label");
 }
 
@@ -1452,9 +1470,9 @@ gtk_about_dialog_get_translator_credits (GtkAboutDialog *about)
  * of the language which is currently used in the user interface.
  * Using gettext(), a simple way to achieve that is to mark the
  * string for translation:
- * <informalexample><programlisting>
+ * |[
  *  gtk_about_dialog_set_translator_credits (about, _("translator-credits"));
- * </programlisting></informalexample>
+ * ]|
  * It is a good idea to use the customary msgid "translator-credits" for this
  * purpose, since translators will already know the purpose of that msgid, and
  * since #GtkAboutDialog will detect if "translator-credits" is untranslated
@@ -1897,7 +1915,7 @@ add_credits_page (GtkAboutDialog *about,
     visited_link_color = default_visited_link_color;
 
   view = gtk_text_view_new ();
-  g_signal_connect_object (about, "style_set",
+  g_signal_connect_object (about, "style-set",
 			   G_CALLBACK (text_view_style_set), view, 0);
   
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
@@ -1907,13 +1925,13 @@ add_credits_page (GtkAboutDialog *about,
   gtk_text_view_set_left_margin (GTK_TEXT_VIEW (view), 8);
   gtk_text_view_set_right_margin (GTK_TEXT_VIEW (view), 8);
 
-  g_signal_connect (view, "key_press_event",
+  g_signal_connect (view, "key-press-event",
                     G_CALLBACK (credits_key_press_event), about);
-  g_signal_connect (view, "event_after",
+  g_signal_connect (view, "event-after",
                     G_CALLBACK (credits_event_after), about);
-  g_signal_connect (view, "motion_notify_event", 
+  g_signal_connect (view, "motion-notify-event", 
                     G_CALLBACK (credits_motion_notify_event), about);
-  g_signal_connect (view, "visibility_notify_event", 
+  g_signal_connect (view, "visibility-notify-event", 
                     G_CALLBACK (credits_visibility_notify_event), about);
 
   sw = gtk_scrolled_window_new (NULL, NULL);
@@ -1964,21 +1982,23 @@ add_credits_page (GtkAboutDialog *about,
 	      gchar *link;
 	      const gchar *link_type;
 	      GtkTextTag *tag;
-	      
-	      gtk_text_buffer_insert_at_cursor (buffer, q0, q1 - q0);
-	      gtk_text_buffer_get_end_iter (buffer, &end);
+
+	      if (*q1 == '<')
+		{
+		  gtk_text_buffer_insert_at_cursor (buffer, q0, (q1 - q0) + 1);
+		  gtk_text_buffer_get_end_iter (buffer, &end);
+		  q1++;
+		  link_type = I_("email");
+		}
+	      else
+		{
+		  gtk_text_buffer_insert_at_cursor (buffer, q0, q1 - q0);
+		  gtk_text_buffer_get_end_iter (buffer, &end);
+		  link_type = I_("url");
+		}
 
 	      q0 = q2;
 
-	      if (*q1 == '<') 
-		{
-		  q1++;
-		  q0++;
-		  link_type = I_("email");
-		}
-	      else 
-		link_type = I_("url");
-	      
 	      link = g_strndup (q1, q2 - q1);
 
 	      if (g_slist_find_custom (priv->visited_links, link, (GCompareFunc)strcmp))
@@ -2283,10 +2303,12 @@ gtk_show_about_dialog (GtkWindow   *parent,
 
       g_object_ref_sink (dialog);
 
-      g_signal_connect (dialog, "delete_event", G_CALLBACK (gtk_widget_hide_on_delete), NULL);
+      g_signal_connect (dialog, "delete-event",
+                        G_CALLBACK (gtk_widget_hide_on_delete), NULL);
 
       /* Close dialog on user response */
-      g_signal_connect (dialog, "response", G_CALLBACK (close_cb), NULL);
+      g_signal_connect (dialog, "response",
+                        G_CALLBACK (close_cb), NULL);
 
       va_start (var_args, first_property_name);
       g_object_set_valist (G_OBJECT (dialog), first_property_name, var_args);
