@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include "gtktoolbutton.h"
 #include "gtkbutton.h"
 #include "gtkhbox.h"
@@ -32,7 +32,7 @@
 #include "gtkvbox.h"
 #include "gtkintl.h"
 #include "gtktoolbar.h"
-#include "gtkiconfactory.h"
+#include "gtkactivatable.h"
 #include "gtkprivate.h"
 #include "gtkalias.h"
 
@@ -78,11 +78,14 @@ static void gtk_tool_button_style_set      (GtkWidget          *widget,
 					    GtkStyle           *prev_style);
 
 static void gtk_tool_button_construct_contents (GtkToolItem *tool_item);
-      
-static GObjectClass *parent_class = NULL;
-static guint         toolbutton_signals[LAST_SIGNAL] = { 0 };
 
-#define GTK_TOOL_BUTTON_GET_PRIVATE(obj)(G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_TOOL_BUTTON, GtkToolButtonPrivate))
+static void gtk_tool_button_activatable_interface_init (GtkActivatableIface  *iface);
+static void gtk_tool_button_activatable_update         (GtkActivatable       *activatable,
+							GtkAction            *action,
+							const gchar          *property_name);
+static void gtk_tool_button_activatable_reset          (GtkActivatable       *activatable,
+							GtkAction            *action);
+
 
 struct _GtkToolButtonPrivate
 {
@@ -95,21 +98,40 @@ struct _GtkToolButtonPrivate
   GtkWidget *icon_widget;
   
   guint use_underline : 1;
+  guint contents_invalid : 1;
 };
 
+static GObjectClass        *parent_class = NULL;
+static GtkActivatableIface *parent_activatable_iface;
+static guint                toolbutton_signals[LAST_SIGNAL] = { 0 };
+
+#define GTK_TOOL_BUTTON_GET_PRIVATE(obj)(G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_TOOL_BUTTON, GtkToolButtonPrivate))
+
 GType
 gtk_tool_button_get_type (void)
 {
-  static GtkType type = 0;
-
+  static GType type = 0;
+  
   if (!type)
-    type = g_type_register_static_simple (GTK_TYPE_TOOL_ITEM,
-					  I_("GtkToolButton"),
-					  sizeof (GtkToolButtonClass),
-					  (GClassInitFunc) gtk_tool_button_class_init,
-					  sizeof (GtkToolButton),
-					  (GInstanceInitFunc) gtk_tool_button_init,
-					  0);
+    {
+      static const GInterfaceInfo activatable_info =
+      {
+        (GInterfaceInitFunc) gtk_tool_button_activatable_interface_init,
+        (GInterfaceFinalizeFunc) NULL,
+        NULL
+      };
+
+      type = g_type_register_static_simple (GTK_TYPE_TOOL_ITEM,
+					    I_("GtkToolButton"),
+					    sizeof (GtkToolButtonClass),
+					    (GClassInitFunc) gtk_tool_button_class_init,
+					    sizeof (GtkToolButton),
+					    (GInstanceInitFunc) gtk_tool_button_init,
+					    0);
+
+      g_type_add_interface_static (type, GTK_TYPE_ACTIVATABLE,
+                                   &activatable_info);
+    }
   return type;
 }
 
@@ -228,7 +250,7 @@ gtk_tool_button_class_init (GtkToolButtonClass *klass)
 							GTK_PARAM_READWRITE));
 
   /**
-   * GtkButton::icon-spacing:
+   * GtkButton:icon-spacing:
    * 
    * Spacing in pixels between the icon and label.
    * 
@@ -270,6 +292,8 @@ gtk_tool_button_init (GtkToolButton      *button,
   
   button->priv = GTK_TOOL_BUTTON_GET_PRIVATE (button);
 
+  button->priv->contents_invalid = TRUE;
+
   gtk_tool_item_set_homogeneous (toolitem, TRUE);
 
   /* create button */
@@ -295,6 +319,8 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
   GtkWidget *box = NULL;
   guint icon_spacing;
 
+  button->priv->contents_invalid = FALSE;
+
   gtk_widget_style_get (GTK_WIDGET (tool_item), 
 			"icon-spacing", &icon_spacing,
 			NULL);
@@ -429,7 +455,7 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
       box = gtk_vbox_new (FALSE, icon_spacing);
       if (icon)
 	gtk_box_pack_start (GTK_BOX (box), icon, TRUE, TRUE, 0);
-      gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);
+      gtk_box_pack_end (GTK_BOX (box), label, FALSE, TRUE, 0);
       gtk_container_add (GTK_CONTAINER (button->priv->button), box);
       break;
 
@@ -488,6 +514,7 @@ gtk_tool_button_set_property (GObject         *object,
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
     }
 }
 
@@ -495,7 +522,10 @@ static void
 gtk_tool_button_property_notify (GObject          *object,
 				 GParamSpec       *pspec)
 {
-  if (strcmp (pspec->name, "is-important") == 0)
+  GtkToolButton *button = GTK_TOOL_BUTTON (object);
+
+  if (button->priv->contents_invalid ||
+      strcmp ("is-important", pspec->name) == 0)
     gtk_tool_button_construct_contents (GTK_TOOL_ITEM (object));
 
   if (parent_class->notify)
@@ -532,6 +562,7 @@ gtk_tool_button_get_property (GObject         *object,
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
     }
 }
 
@@ -570,6 +601,12 @@ clone_image_menu_size (GtkImage *image, GtkSettings *settings)
       gtk_image_get_icon_set (image, &icon_set, NULL);
       return gtk_image_new_from_icon_set (icon_set, GTK_ICON_SIZE_MENU);
     }
+  else if (storage_type == GTK_IMAGE_GICON)
+    {
+      GIcon *icon;
+      gtk_image_get_gicon (image, &icon, NULL);
+      return gtk_image_new_from_gicon (icon, GTK_ICON_SIZE_MENU);
+    }
   else if (storage_type == GTK_IMAGE_PIXBUF)
     {
       gint width, height;
@@ -605,7 +642,10 @@ gtk_tool_button_create_menu_proxy (GtkToolItem *item)
   gboolean use_mnemonic = TRUE;
   const char *label;
 
-  if (button->priv->label_widget && GTK_IS_LABEL (button->priv->label_widget))
+  if (_gtk_tool_item_create_menu_proxy (item))
+    return TRUE;
+ 
+  if (GTK_IS_LABEL (button->priv->label_widget))
     {
       label = gtk_label_get_label (GTK_LABEL (button->priv->label_widget));
       use_mnemonic = gtk_label_get_use_underline (GTK_LABEL (button->priv->label_widget));
@@ -629,7 +669,7 @@ gtk_tool_button_create_menu_proxy (GtkToolItem *item)
   else
     menu_item = gtk_image_menu_item_new_with_label (label);
 
-  if (button->priv->icon_widget && GTK_IS_IMAGE (button->priv->icon_widget))
+  if (GTK_IS_IMAGE (button->priv->icon_widget))
     {
       menu_image = clone_image_menu_size (GTK_IMAGE (button->priv->icon_widget),
 					  gtk_widget_get_settings (GTK_WIDGET (button)));
@@ -657,6 +697,13 @@ static void
 button_clicked (GtkWidget     *widget,
 		GtkToolButton *button)
 {
+  GtkAction *action;
+
+  action = gtk_activatable_get_related_action (GTK_ACTIVATABLE (button));
+  
+  if (action)
+    gtk_action_activate (action);
+
   g_signal_emit_by_name (button, "clicked");
 }
 
@@ -689,6 +736,126 @@ gtk_tool_button_style_set (GtkWidget *widget,
   gtk_tool_button_update_icon_spacing (GTK_TOOL_BUTTON (widget));
 }
 
+static void 
+gtk_tool_button_activatable_interface_init (GtkActivatableIface  *iface)
+{
+  parent_activatable_iface = g_type_interface_peek_parent (iface);
+  iface->update = gtk_tool_button_activatable_update;
+  iface->reset = gtk_tool_button_activatable_reset;
+}
+
+static void
+gtk_tool_button_activatable_update (GtkActivatable       *activatable,
+				    GtkAction            *action,
+				    const gchar          *property_name)
+{
+  GtkToolButton *button;
+  GtkWidget *image;
+
+  parent_activatable_iface->update (activatable, action, property_name);
+
+  if (!gtk_activatable_get_use_action_appearance (activatable))
+    return;
+
+  button = GTK_TOOL_BUTTON (activatable);
+  
+  if (strcmp (property_name, "short-label") == 0)
+    {
+      if (!gtk_action_get_stock_id (action) &&
+	  !gtk_action_get_icon_name (action))
+	{
+	  gtk_tool_button_set_use_underline (button, TRUE);
+	  gtk_tool_button_set_label (button, gtk_action_get_short_label (action));
+	}
+    }
+  else if (strcmp (property_name, "stock-id") == 0)
+    {
+      if (gtk_action_get_stock_id (action))
+	{	
+	  gtk_tool_button_set_label (button, NULL);
+	  gtk_tool_button_set_icon_name (button, NULL);
+	}
+      gtk_tool_button_set_icon_widget (button, NULL);
+      gtk_tool_button_set_stock_id (button, gtk_action_get_stock_id (action));
+    }
+  else if (strcmp (property_name, "gicon") == 0)
+    {
+      const gchar *stock_id = gtk_action_get_stock_id (action);
+      GIcon *icon = gtk_action_get_gicon (action);
+      GtkIconSize icon_size = GTK_ICON_SIZE_BUTTON;
+
+      if ((stock_id && gtk_icon_factory_lookup_default (stock_id)) || !icon)
+	image = NULL;
+      else 
+	{   
+	  image = gtk_tool_button_get_icon_widget (button);
+	  icon_size = gtk_tool_item_get_icon_size (GTK_TOOL_ITEM (button));
+
+	  if (!image)
+	    image = gtk_image_new ();
+	}
+
+      gtk_tool_button_set_icon_widget (button, image);
+      gtk_image_set_from_gicon (GTK_IMAGE (image), icon, icon_size);
+
+    }
+  else if (strcmp (property_name, "icon-name") == 0)
+    {
+      if (gtk_action_get_icon_name (action))
+	{	
+	  gtk_tool_button_set_label (button, NULL);
+	  gtk_tool_button_set_stock_id (button, NULL);
+	}
+      gtk_tool_button_set_icon_name (button, gtk_action_get_icon_name (action));
+    }
+}
+
+static void
+gtk_tool_button_activatable_reset (GtkActivatable       *activatable,
+				   GtkAction            *action)
+{
+  GtkToolButton *button;
+  GIcon         *icon;
+  const gchar   *stock_id;
+
+  parent_activatable_iface->reset (activatable, action);
+
+  if (!action)
+    return;
+
+  if (!gtk_activatable_get_use_action_appearance (activatable))
+    return;
+
+  button = GTK_TOOL_BUTTON (activatable);
+  stock_id = gtk_action_get_stock_id (action);
+
+  gtk_tool_button_set_label (button, gtk_action_get_short_label (action));
+  gtk_tool_button_set_use_underline (button, TRUE);
+  gtk_tool_button_set_stock_id (button, stock_id);
+  gtk_tool_button_set_icon_name (button, gtk_action_get_icon_name (action));
+
+  if (stock_id && gtk_icon_factory_lookup_default (stock_id))
+      gtk_tool_button_set_icon_widget (button, NULL);
+  else if ((icon = gtk_action_get_gicon (action)) != NULL)
+    {
+      GtkIconSize icon_size = gtk_tool_item_get_icon_size (GTK_TOOL_ITEM (button));
+      GtkWidget  *image = gtk_tool_button_get_icon_widget (button);
+      
+      if (!image)
+	{
+	  image = gtk_image_new ();
+	  gtk_widget_show (image);
+	  gtk_tool_button_set_icon_widget (button, image);
+	}
+
+      gtk_image_set_from_gicon (GTK_IMAGE (image), icon, icon_size);
+    }
+  else if (gtk_action_get_icon_name (action))
+    gtk_tool_button_set_icon_name (button, gtk_action_get_icon_name (action));
+  else
+    gtk_tool_button_set_label (button, gtk_action_get_short_label (action));
+}
+
 /**
  * gtk_tool_button_new_from_stock:
  * @stock_id: the name of the stock item 
@@ -736,13 +903,9 @@ gtk_tool_button_new (GtkWidget	 *icon_widget,
   GtkToolButton *button;
 
   button = g_object_new (GTK_TYPE_TOOL_BUTTON,
+                         "label", label,
+                         "icon-widget", icon_widget,
 			 NULL);
-  
-  if (label)
-    gtk_tool_button_set_label (button, label);
-
-  if (icon_widget)
-    gtk_tool_button_set_icon_widget (button, icon_widget);
 
   return GTK_TOOL_ITEM (button);  
 }
@@ -771,12 +934,11 @@ gtk_tool_button_set_label (GtkToolButton *button,
   old_label = button->priv->label_text;
 
   button->priv->label_text = g_strdup (label);
-  gtk_tool_button_construct_contents (GTK_TOOL_ITEM (button));
-      
-  g_object_notify (G_OBJECT (button), "label");
+  button->priv->contents_invalid = TRUE;     
 
-  if (old_label)
-    g_free (old_label);
+  g_free (old_label);
+ 
+  g_object_notify (G_OBJECT (button), "label");
 }
 
 /**
@@ -826,8 +988,7 @@ gtk_tool_button_set_use_underline (GtkToolButton *button,
   if (use_underline != button->priv->use_underline)
     {
       button->priv->use_underline = use_underline;
-
-      gtk_tool_button_construct_contents (GTK_TOOL_ITEM (button));
+      button->priv->contents_invalid = TRUE;
 
       g_object_notify (G_OBJECT (button), "use-underline");
     }
@@ -875,11 +1036,11 @@ gtk_tool_button_set_stock_id (GtkToolButton *button,
   old_stock_id = button->priv->stock_id;
 
   button->priv->stock_id = g_strdup (stock_id);
-  gtk_tool_button_construct_contents (GTK_TOOL_ITEM (button));
-  
-  g_object_notify (G_OBJECT (button), "stock-id");
+  button->priv->contents_invalid = TRUE;
 
   g_free (old_stock_id);
+  
+  g_object_notify (G_OBJECT (button), "stock-id");
 }
 
 /**
@@ -925,11 +1086,11 @@ gtk_tool_button_set_icon_name (GtkToolButton *button,
   old_icon_name = button->priv->icon_name;
 
   button->priv->icon_name = g_strdup (icon_name);
-  gtk_tool_button_construct_contents (GTK_TOOL_ITEM (button));
-
-  g_object_notify (G_OBJECT (button), "icon-name");
+  button->priv->contents_invalid = TRUE; 
 
   g_free (old_icon_name);
+
+  g_object_notify (G_OBJECT (button), "icon-name");
 }
 
 /**
@@ -975,22 +1136,17 @@ gtk_tool_button_set_icon_widget (GtkToolButton *button,
       if (button->priv->icon_widget)
 	{
 	  if (button->priv->icon_widget->parent)
-	    {
-	      gtk_container_remove (GTK_CONTAINER (button->priv->icon_widget->parent),
+	    gtk_container_remove (GTK_CONTAINER (button->priv->icon_widget->parent),
 				    button->priv->icon_widget);
-	    }
 
 	  g_object_unref (button->priv->icon_widget);
 	}
       
       if (icon_widget)
-	{
-	  g_object_ref_sink (icon_widget);
-	}
+	g_object_ref_sink (icon_widget);
 
       button->priv->icon_widget = icon_widget;
-
-      gtk_tool_button_construct_contents (GTK_TOOL_ITEM (button));
+      button->priv->contents_invalid = TRUE;
       
       g_object_notify (G_OBJECT (button), "icon-widget");
     }
@@ -1021,22 +1177,17 @@ gtk_tool_button_set_label_widget (GtkToolButton *button,
       if (button->priv->label_widget)
 	{
 	  if (button->priv->label_widget->parent)
-	    {
-	      gtk_container_remove (GTK_CONTAINER (button->priv->label_widget->parent),
-				    button->priv->label_widget);
-	    }
+	    gtk_container_remove (GTK_CONTAINER (button->priv->label_widget->parent),
+		    	          button->priv->label_widget);
 	  
 	  g_object_unref (button->priv->label_widget);
 	}
       
       if (label_widget)
-	{
-	  g_object_ref_sink (label_widget);
-	}
+	g_object_ref_sink (label_widget);
 
       button->priv->label_widget = label_widget;
-
-      gtk_tool_button_construct_contents (GTK_TOOL_ITEM (button));
+      button->priv->contents_invalid = TRUE;
       
       g_object_notify (G_OBJECT (button), "label-widget");
     }
